import numpy as np
import scipy.stats as stats

def infer_binomial(data, prior_alpha=2, prior_beta=2):
    """
    Perform Bayesian inference for a binomial likelihood with a beta prior.
    
    :param data: List of observed data (1 for success, 0 for failure).
    :param prior_alpha: Alpha parameter of the Beta prior. Default is 2.
    :param prior_beta: Beta parameter of the Beta prior. Default is 2.
    :return: Posterior alpha and beta parameters.
    """
    heads = sum(data)
    tails = len(data) - heads
    posterior_alpha = prior_alpha + heads
    posterior_beta = prior_beta + tails
    return posterior_alpha, posterior_beta

def infer_poisson(data, prior_alpha=1, prior_beta=1):
    """
    Perform Bayesian inference for a Poisson likelihood with a gamma prior.
    
    :param data: List of observed data (counts).
    :param prior_alpha: Shape parameter of the Gamma prior. Default is 1.
    :param prior_beta: Rate parameter of the Gamma prior. Default is 1.
    :return: Posterior alpha and beta parameters.
    """
    total_counts = sum(data)
    n = len(data)
    posterior_alpha = prior_alpha + total_counts
    posterior_beta = prior_beta + n
    return posterior_alpha, posterior_beta

def infer_normal_known_variance(data, known_variance, prior_mean=0, prior_precision=1):
    """
    Perform Bayesian inference for a normal likelihood with a normal prior and known variance.
    
    :param data: List of observed data.
    :param known_variance: Known variance of the likelihood.
    :param prior_mean: Mean of the Normal prior. Default is 0.
    :param prior_precision: Precision (1/variance) of the Normal prior. Default is 1.
    :return: Posterior mean and precision.
    """
    n = len(data)
    sample_mean = np.mean(data)
    posterior_precision = prior_precision + n / known_variance
    posterior_mean = (prior_precision * prior_mean + n * sample_mean / known_variance) / posterior_precision
    return posterior_mean, posterior_precision

def infer_normal_unknown_variance(data, prior_mean=0, prior_precision=1, prior_df=2, prior_scale=1):
    """
    Perform Bayesian inference for a normal likelihood with a normal-gamma prior.
    
    :param data: List of observed data.
    :param prior_mean: Mean of the Normal prior. Default is 0.
    :param prior_precision: Precision (1/variance) of the Normal prior. Default is 1.
    :param prior_df: Degrees of freedom of the Gamma prior. Default is 2.
    :param prior_scale: Scale parameter of the Gamma prior. Default is 1.
    :return: Posterior mean, precision, degrees of freedom, and scale.
    """
    n = len(data)
    sample_mean = np.mean(data)
    sample_variance = np.var(data, ddof=1)
    posterior_precision = prior_precision + n
    posterior_mean = (prior_precision * prior_mean + n * sample_mean) / posterior_precision
    posterior_df = prior_df + n
    posterior_scale = (prior_df * prior_scale + (n - 1) * sample_variance + 
                       (prior_precision * n * (sample_mean - prior_mean)**2) / posterior_precision) / posterior_df
    return posterior_mean, posterior_precision, posterior_df, posterior_scale



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import sys
from setuptools import find_packages, setup

sys.path.insert(0, os.path.join(os.path.dirname(__file__), 'bayinf'))
from version import __version__ as version

setup(
    name='bayinf',
    version=version,
    description='Bayesian Inference',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='Zhiqing Xiao',
    author_email='xzq.xiaozhiqing@gmail.com',
    url='http://github.com/zhiqingxiao/bayinf/',
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        'Programming Language :: Python',
        'Intended Audience :: Science/Research',
    ],
)



import numpy as np
import matplotlib.pyplot as plt
import scipy.stats as stats

def beta(alpha, beta, title):
    """
    Plot the Beta distribution with given alpha and beta parameters.
    
    :param alpha: Alpha parameter of the Beta distribution.
    :param beta: Beta parameter of the Beta distribution.
    :param title: Title of the plot.
    """
    x = np.linspace(0, 1, 1000)
    y = stats.beta.pdf(x, alpha, beta)
    plt.plot(x, y, label=f'Beta({alpha}, {beta})')
    plt.title(title)
    plt.xlabel('θ (Probability of Success)')
    plt.ylabel('Density')
    plt.legend()
    plt.show()

def gamma(alpha, beta, title):
    """
    Plot the Gamma distribution with given alpha and beta parameters.
    
    :param alpha: Shape parameter of the Gamma distribution.
    :param beta: Rate parameter of the Gamma distribution.
    :param title: Title of the plot.
    """
    x = np.linspace(0, 10, 1000)
    y = stats.gamma.pdf(x, alpha, scale=1/beta)
    plt.plot(x, y, label=f'Gamma({alpha}, {beta})')
    plt.title(title)
    plt.xlabel('λ (Rate Parameter)')
    plt.ylabel('Density')
    plt.legend()
    plt.show()

def normal(mean, precision, title):
    """
    Plot the Normal distribution with given mean and precision.
    
    :param mean: Mean of the Normal distribution.
    :param precision: Precision (1/variance) of the Normal distribution.
    :param title: Title of the plot.
    """
    variance = 1 / precision
    std_dev = np.sqrt(variance)
    x = np.linspace(mean - 3 * std_dev, mean + 3 * std_dev, 1000)
    y = stats.norm.pdf(x, mean, std_dev)
    plt.plot(x, y, label=f'Normal({mean}, {1/precision})')
    plt.title(title)
    plt.xlabel('μ (Mean)')
    plt.ylabel('Density')
    plt.legend()
    plt.show()
